"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var controller_factory_1 = require("./controller-factory");
var validator_1 = require("../validator");
var binder_1 = require("../binder");
function createController(option, controllerInfo, parameters) {
    var validator = new validator_1.ValidatorImpl(option.metaDataStorage, option.validators);
    validator.setValue(parameters, controllerInfo.classMetaData, controllerInfo.methodMetaData.name);
    var controller = controller_factory_1.ControllerFactory.resolve(controllerInfo, option.dependencyResolver);
    controller.validator = validator;
    return controller;
}
var MiddlewareInvocation = (function (_super) {
    tslib_1.__extends(MiddlewareInvocation, _super);
    function MiddlewareInvocation(invocation, context, middleware) {
        var _this = _super.call(this) || this;
        _this.invocation = invocation;
        _this.context = context;
        _this.middleware = middleware;
        _this.controllerInfo = invocation.controllerInfo;
        _this.middlewares = invocation.middlewares;
        return _this;
    }
    MiddlewareInvocation.prototype.proceed = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this.middleware.execute(this.context, this.invocation)];
            });
        });
    };
    return MiddlewareInvocation;
}(Core.Invocation));
exports.MiddlewareInvocation = MiddlewareInvocation;
var ErrorInvocation = (function (_super) {
    tslib_1.__extends(ErrorInvocation, _super);
    function ErrorInvocation(error) {
        var _this = _super.call(this) || this;
        _this.error = error;
        return _this;
    }
    ErrorInvocation.prototype.proceed = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                throw this.error;
            });
        });
    };
    return ErrorInvocation;
}(Core.Invocation));
exports.ErrorInvocation = ErrorInvocation;
var HttpControllerInvocation = (function (_super) {
    tslib_1.__extends(HttpControllerInvocation, _super);
    function HttpControllerInvocation(option, request, controllerInfo) {
        var _this = _super.call(this) || this;
        _this.option = option;
        _this.request = request;
        _this.controllerInfo = controllerInfo;
        return _this;
    }
    HttpControllerInvocation.prototype.proceed = function () {
        var binder = new binder_1.ParameterBinder(this.controllerInfo, this.option.pathResolver);
        var parameters = binder.getParameters(this.request);
        var controller = createController(this.option, this.controllerInfo, parameters);
        controller.context = this.request;
        var method = controller[this.controllerInfo.methodMetaData.name];
        var result;
        if (this.option.autoValidation && !controller.validator.isValid())
            result = new Core.ActionResult(controller.validator.getValidationErrors(), 400, "application/json");
        else
            result = method.apply(controller, parameters);
        return this.createResult(result);
    };
    HttpControllerInvocation.prototype.createResult = function (result) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var awaitedResult;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.resolve(result)];
                    case 1:
                        awaitedResult = _a.sent();
                        if (awaitedResult instanceof Core.ActionResult)
                            return [2 /*return*/, awaitedResult];
                        if (this.controllerInfo.classMetaData.baseClass == "ApiController") {
                            return [2 /*return*/, new Core.ActionResult(awaitedResult, 200, "application/json")];
                        }
                        return [2 /*return*/, new Core.ActionResult(awaitedResult, 200, "text/html")];
                }
            });
        });
    };
    return HttpControllerInvocation;
}(Core.Invocation));
exports.HttpControllerInvocation = HttpControllerInvocation;
var SocketControllerInvocation = (function (_super) {
    tslib_1.__extends(SocketControllerInvocation, _super);
    function SocketControllerInvocation(option, socket, controllerInfo, msg) {
        var _this = _super.call(this) || this;
        _this.option = option;
        _this.socket = socket;
        _this.controllerInfo = controllerInfo;
        _this.msg = msg;
        return _this;
    }
    SocketControllerInvocation.prototype.proceed = function () {
        var controller = createController(this.option, this.controllerInfo, [this.msg]);
        controller.context = this.socket;
        var method = controller[this.controllerInfo.methodMetaData.name];
        var result;
        if (this.option.autoValidation && !controller.validator.isValid())
            result = new Core.ActionResult(controller.validator.getValidationErrors(), 400);
        else
            result = method.apply(controller, [this.msg]);
        return this.createResult(result);
    };
    SocketControllerInvocation.prototype.createResult = function (result) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var awaitedResult;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Promise.resolve(result)];
                    case 1:
                        awaitedResult = _a.sent();
                        if (awaitedResult instanceof Core.ActionResult)
                            return [2 /*return*/, awaitedResult];
                        return [2 /*return*/, new Core.ActionResult(result, 200)];
                }
            });
        });
    };
    return SocketControllerInvocation;
}(Core.Invocation));
exports.SocketControllerInvocation = SocketControllerInvocation;
