"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var invocations_1 = require("./invocations");
var framework_1 = require("../framework");
var middleware_factory_1 = require("./middleware-factory");
var controller_factory_1 = require("./controller-factory");
var Invoker = (function () {
    function Invoker(option) {
        this.option = option;
    }
    Invoker.prototype.invoke = function (context, controllerInvocation) {
        var invocation = controllerInvocation;
        this.getMiddlewares(controllerInvocation.controllerInfo).forEach(function (middleware) {
            invocation = new invocations_1.MiddlewareInvocation(invocation, context, middleware);
        });
        return invocation.proceed();
    };
    Invoker.prototype.getMiddlewares = function (routeInfo) {
        var globalMiddlewares = (this.option.middlewares || []).slice().reverse();
        if (routeInfo) {
            var controller = controller_factory_1.ControllerFactory.resolve(routeInfo, this.option.dependencyResolver);
            var middlewares = middleware_factory_1.MiddlewareFactory.resolve(framework_1.MiddlewareDecorator.getMiddlewares(controller), this.option.dependencyResolver);
            middlewares = globalMiddlewares.concat(middlewares);
            return middlewares.concat(middleware_factory_1.MiddlewareFactory.resolve(framework_1.MiddlewareDecorator.getMiddlewares(controller, routeInfo.methodMetaData.name), this.option.dependencyResolver));
        }
        else {
            return globalMiddlewares;
        }
    };
    return Invoker;
}());
exports.Invoker = Invoker;
